"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

require("core-js/modules/es6.object.define-property");

require("core-js/modules/es6.array.index-of");

require("core-js/modules/es6.function.name");

require("core-js/modules/es6.regexp.split");

require("core-js/modules/es6.string.repeat");

require("core-js/modules/es6.array.reduce");

require("core-js/modules/es6.regexp.replace");

var _pathToRegexp = _interopRequireDefault(require("path-to-regexp"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

/**
 * This class defines a single route. It supports dynamic
 * **url segments** associated **data**
 * and **named routes**.
 *
 * Normally you don't need to create an instance of this class.
 * Use the Router singleton to create a route.
 *
 * Example: `router.add(name, page)`
 *
 * @class Route
 * @constructor
 */
var Route =
/*#__PURE__*/
function () {
  function Route(router, route, page) {
    _classCallCheck(this, Route);

    this._instantiate(router, route, page);

    this._makeRoutePattern();
  }
  /**
   * @description
   * Define domain for the route. If domain is defined
   * then route will only resolve when domain matches.
   * Also see [Group.domain](https://github.com/samueljoos/next-avenues/blob/master/docs/group.md#domaindomain).
   *
   * @function domain
   *
   * @param  {string}  domain The domain template string.
   * @returns {Route}
   *
   * @example
   * router
   *   .add('/', 'home')
   *   .domain('blog.next-avenues.org')
   */


  _createClass(Route, [{
    key: "domain",
    value: function domain(_domain) {
      this.domainKeys = [];
      var cleanDomain = "".concat(_domain.replace(/^\/|\/$/g, ''));
      this.forDomain = (0, _pathToRegexp.default)(cleanDomain, this.domainKeys);
      this._domain = cleanDomain;
      return this;
    }
    /**
     * @description
     * Define a name as an identifier for your route.
     * The main benefit of using a name is that you can make change the route
     * without having to refactor all your route references.
     *
     * @function as
     *
     * @param  {string} name The route name.
     *
     * @returns {Route}
     *
     * @example
     * router
     *   .add('/', 'home')
     *   .as('name')
     */

  }, {
    key: "as",
    value: function as(name) {
      this.name = name;
      return this;
    }
    /**
     * @description
     * Prefix the route with some string.
     * Generally used via [Group.prefix](https://github.com/samueljoos/next-avenues/blob/master/docs/group.md#prefixprefix) to prefix a bunch of routes.
     *
     * @function prefix
     *
     * @param  {string} prefix
     *
     * @returns {Route}
     *
     * @example
     * router
     *   .add('/articles', 'articles')
     *   .prefix('api/v1')
     */

  }, {
    key: "prefix",
    value: function prefix(_prefix) {
      var cleanPrefix = "/".concat(_prefix.replace(/^\/|\/$/g, ''));
      this._route = this._route === '/' ? cleanPrefix : "".concat(cleanPrefix).concat(this._route);

      this._makeRoutePattern();

      return this;
    }
    /**
     * @description
     * Associate some static (meta)data with a route.
     * This data will be available when you call [Router.getCurrentRoute](https://github.com/samueljoos/next-avenues/blob/master/docs/router.md#getcurrentroute).
     * Also see [Group.data](https://github.com/samueljoos/next-avenues/blob/master/docs/group.md#datadata).
     *
     * Example: This can be handy when you want to create multilingual domain setup.
     *
     * @function data
     *
     * @param  {Object} data Static data.
     *
     * @returns {Route}
     *
     * @example
     * const multilingualRoutes = () => {
     *     router.add('/path', 'page')
     * }
     * router.group(multilingualRoutes).domain('myfrenchwebsite.fr').data({lang: 'fr'})
     * router.group(multilingualRoutes).domain('mydutchwebsite.nl').data({lang: 'nl'})
     */

  }, {
    key: "data",
    value: function data(_data) {
      this.data = _data;
      return this;
    }
    /**
     * @description
     * Resolves the url by matching it against
     * the registered route and verbs. It will
     * return an empty object when the url
     * doesn't match to this route.
     *
     * @function resolve
     *
     * @param  {string} url The url string.
     * @param  {string} host The domain string.
     *
     * @returns {Object}
     */

  }, {
    key: "resolve",
    value: function resolve(url, host) {
      /*
      * Check for matching subdomains
      */
      var subdomains = this._getSubDomains(host);

      if (this.forDomain && !subdomains) {
        return null;
      }
      /*
      * Nothing needs processing, since the route
      * and the url are same.
      */


      if (this._route === url) {
        return {
          url: url,
          params: {},
          subdomains: subdomains || {}
        };
      }
      /*
      * Get route tokens if matched otherwise
      * return null.
      */


      var tokens = this._regexp.exec(url);

      if (!tokens) {
        return null;
      }

      var params = this._keys.reduce(function (result, key, index) {
        var value = tokens[index + 1] || null;
        value = key.repeat && value ? value.split('/') : value;
        result[key.name] = value;
        return result;
      }, {});

      return {
        url: url,
        params: params,
        subdomains: subdomains || {}
      };
    }
    /**
     * @description
     * Create an url based on the params and options provided.
     *
     * @function getUrl
     *
     * @param {Object.<string, string>} [params] Data to build the url path.
     * @param {{protocol: ?string, domain: ?string, query:?Object.<string, string>}} options Options object.
     *
     * @returns {string}
     */

  }, {
    key: "getUrl",
    value: function getUrl() {
      var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var compiledDomain;

      if (this._domain) {
        if (this.domainKeys.length) {
          compiledDomain = _pathToRegexp.default.compile(this._domain)(params);
        } else {
          compiledDomain = this._domain;
        }
      }

      var compiledRoute = _pathToRegexp.default.compile(this._route)(params || {});
      /**
      * When domain exists, build a complete url over creating
      * a relative URL.
      */


      compiledDomain = options.domain || compiledDomain;

      if (compiledDomain && compiledDomain !== this.router.domain) {
        var protocol = options.protocol || this.router.protocol;
        var port = this.router.port ? ':' + this.router.port : '';
        return "".concat(protocol, "://").concat(compiledDomain).concat(port).concat(compiledRoute);
      }

      return "".concat(compiledRoute).concat(options.query ? (0, _utils.toQuerystring)(options.query || {}) : '');
    }
    /**
     * @description
     * Get the Next.js page component name.
     *
     * @function getPage
     *
     * @returns {string}
     */

  }, {
    key: "getPage",
    value: function getPage() {
      return this.page;
    }
    /**
     * @description
     * Get the next/link component props for this route.
     * Generally you don't need this function and it's
     * beter to use the @Ref(Router/Link) component.
     *
     * @function getNextLinkProps
     *
     * @param {Object.<string, string>} [params] Data to build the url path.
     * @param {{protocol: ?string, domain: ?string, query:?Object.<string, string>}} [options] Options object.
     *
     * @returns {Object}
     */

  }, {
    key: "getNextLinkProps",
    value: function getNextLinkProps() {
      var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var url = this.getUrl(params, options);
      var protocol = options.protocol || this.router.protocol;

      if (url.indexOf(protocol) === 0) {
        return {
          href: url
        };
      }

      return {
        as: url,
        href: '/' + this.page
      };
    }
    /**
     * @description
     * Returns a JSON representation of the route.
     *
     * @function toJSON
     *
     * @returns {Object}
     */

  }, {
    key: "toJSON",
    value: function toJSON() {
      return {
        route: this._route,
        name: this.name,
        domain: this._domain,
        data: this.data,
        page: this.page
      };
    } //////
    // Private functions
    //////

    /**
     * @description
     * Validates the route to make sure it is a
     * valid string
     *
     * @function _validateRoute
     *
     * @param {string} route The route template path.
     *
     * @returns {void}
     *
     * @private
     */

  }, {
    key: "_validateRoute",
    value: function _validateRoute(route) {
      if (typeof route !== 'string') {
        throw (0, _utils.errorMessage)('Cannot instantiate route without a valid url string', route);
      }
    }
    /**
     * @description
     * Instantiate private properties on the route instance
     *
     * @function _instantiate
     *
     * @param  {Router} router Reference to the [Router](https://github.com/samueljoos/next-avenues/blob/master/docs/router.md).
     * @param  {string} route The route template path.
     * @param  {string} page The Next.js page component name.
     *
     * @private
     */

  }, {
    key: "_instantiate",
    value: function _instantiate(router, route, page) {
      this.router = router;

      this._validateRoute(route);

      var routenName = "/".concat(route.replace(/^\/|\/$/g, '')); // private properties

      this._route = routenName === '/*' ? '/(.*)' : routenName;
      this._keys = []; // public properties

      this.name = routenName;
      this.page = page;
      this.forDomain = null;
      this.domainKeys = [];
    }
    /**
     * @description
     * Make the regexp pattern for the route. Later this
     * expression is used to match urls.
     *
     * @function _makeRoutePattern
     *
     * @private
     */

  }, {
    key: "_makeRoutePattern",
    value: function _makeRoutePattern() {
      this._keys = [];
      this._regexp = (0, _pathToRegexp.default)(this._route, this._keys);
    }
    /**
     * @description
     * Returns an object of dynamic domains for a given
     * route.
     *
     * @function _getSubDomains
     *
     * @param {string} host The domain template string.
     *
     * @returns {Object|Null}
     *
     * @private
     */

  }, {
    key: "_getSubDomains",
    value: function _getSubDomains(host) {
      if (!this.forDomain) {
        return null;
      }

      var domainTokens = this.forDomain.exec(host);

      if (!domainTokens) {
        return null;
      }

      return this.domainKeys.reduce(function (result, key, index) {
        var value = domainTokens[index + 1] || null;
        result[key.name] = value;
        return result;
      }, {});
    }
  }]);

  return Route;
}();

var _default = Route;
exports.default = _default;